// Fragment shader for per-pixel Phong interpolation and shading.

// The "varying" keyword means that the parameter's value is interpolated
// between the nearby vertices.
varying vec3 N;
varying vec3 v;

void main()
{
    // The scene's ambient light.
    vec4 ambient = gl_LightModel.ambient * gl_FrontMaterial.ambient;

	// The normal vectors is generally not normalized after being
	// interpolated across a triangle.  Here we normalize it.
	vec3 Normal = normalize(N);

	// Since the vertex is in eye space, the direction to the
	// viewer is simply the normalized vector from v to the
	// origin.
	vec3 Viewer = -normalize(v);

	// Get the lighting direction and normalize it.
	vec3 Light  = normalize(gl_LightSource[0].position.xyz);

	// Compute halfway vector
	vec3 Half = normalize(Viewer+Light);

	// Compute factor to prevent light leakage from below the
	// surface
	float B = 1.0;
	if(dot(Normal, Light)<0.0) B = 0.0;

	// Compute geometric terms of diffuse and specular
	float diffuseShade = max(dot(Normal, Light), 0.0);
	float specularShade = 
	  B * pow(max(dot(Half, Normal), 0.0), gl_FrontMaterial.shininess);

    // Compute the direction of point light
	vec3 lightDir = gl_LightSource[1].position.xyz - v;

	// Compute the distance to the point light
	float dist = length(lightDir);

	// Compute the dot product between normal and point light direction
	float NdotPl = max(dot(Normal, normalize(lightDir)), 0.0);

	// Compute the attenuation
	float att = 1.0 / (gl_LightSource[1].constantAttenuation + 
					gl_LightSource[1].linearAttenuation * dist + 
					gl_LightSource[1].quadraticAttenuation * dist * dist);

	vec4 PointLightColor = att * gl_FrontLightProduct[1].ambient;

	if(NdotPl > 0.0){
		
		PointLightColor += att * (gl_FrontLightProduct[1].diffuse * NdotPl);

		vec3 halfV = normalize(Viewer+normalize(lightDir));
		float NdotHV = max(dot(Normal,halfV), 0.0);
		//PointLightColor += att * pow(NdotHV,gl_FrontMaterial.shininess);
	}

	// Compute product of geometric terms with material and
	// lighting values
	vec4 diffuse = diffuseShade * gl_FrontLightProduct[0].diffuse;
	vec4 specular = specularShade * gl_FrontLightProduct[0].specular;
	ambient += gl_FrontLightProduct[0].ambient;

	// Assign final colors
	gl_FragColor = ambient + diffuse + PointLightColor;

	int r = int(gl_FragColor.x * 255); 
	int g = int(gl_FragColor.y * 255); 
	int b = int(gl_FragColor.z * 255);

	// allow ~4 values per color channel
	r = (r/30) * 30;
	g = (g/30) * 30;
	b = (b/30) * 30;

	gl_FragColor.x = float(r / 255.0);
	gl_FragColor.y = float(g / 255.0);
	gl_FragColor.z = float(b / 255.0);
}